import datetime
from functools import singledispatch
from src.convert.collection_converter import collection_converter
from src.convert.dict_converter import dict_converter
from src.models.abstract_reference import abstract_reference
from src.convert.basic_converter import basic_converter
from src.convert.datetime_converter import datetime_converter
from src.convert.model_converter import model_converter


class converter_factory:
    @staticmethod
    def convert(obj):
        return converter_factory.create(obj).convert(obj)

    @singledispatch
    @staticmethod
    def create(obj):
        return basic_converter()

    @create.register
    def create_set(obj: set):
        return collection_converter()

    @create.register
    def create_list(obj: list):
        return collection_converter()

    @create.register
    def create_dict(obj: dict):
        return dict_converter()

    @create.register
    def create_model(obj: abstract_reference):
        return model_converter()

    @create.register
    def create_datetime(obj: datetime.datetime):
        return datetime_converter()
