from src.models.abstract_reference import abstract_reference
from src.validation.validator import validator


class measurement_unit_model(abstract_reference):
    # Базовая единица измерения
    __base_measurement_unit = None
    # Базовый коэффициент
    __base_coefficient = None

    # Валидатор
    __vtor = validator()

    def __init__(self, name, base_coefficient=1, base_measurement=None):
        """
        Args:
            base_measurement (measurement_unit_model): Базовая единица измерения
            base_coefficint (number): Базовый коэффициент
        """

        self.__vtor.check_type_any(base_measurement, measurement_unit_model, None)
        self.__vtor.check_number(base_coefficient)

        self.__base_measurement_unit = base_measurement
        self.__base_coefficient = base_coefficient

        super().__init__(name)

    @property
    def base_measurement_unit(self):
        """
        Базовая единица измерения
        """

        if self.__base_measurement_unit is None:
            return None

        return self.__base_measurement_unit

    @base_measurement_unit.setter
    def base_measurement_unit(self, value):
        """
            Базовая единица измерения
        Args:
            value (base_measurement_unit/None): Базовая единица измерения
        Raises:
            argument_exception: Несоответствие типа аргумента
        """

        self.__vtor.check_type_any(value, measurement_unit_model, None)

        self.__base_measurement_unit = value

    @property
    def base_coefficient(self):
        """
        Коэффициент относительно базовой единицы измерения
        """

        return self.__base_coefficient

    @base_coefficient.setter
    def base_coefficient(self, value):
        """
            Коэффициент относительно базовой единицы измерения
        Args:
            value (Number): Базовый коэффициент
        Raises:
            argument_exception: Аргумент не является числом
        """

        self.__vtor.check_number(value)

        self.__base_coefficient = value

    def to_base_unit(self, value):
        """
        Значение в базовой единице измерения
        """

        return value * self.base_coefficient

    @staticmethod
    def create_g():
        """
            Создать единицу измерения грамм

        Returns:
            Модель грамма
        """
        item = measurement_unit_model("грамм")
        return item

    @staticmethod
    def create_kg():
        """
            Создать единицу килограмм
        Returns:
            Модель килограмма
        """
        g = measurement_unit_model.create_g()
        item = measurement_unit_model("киллограмм", 1000, g)
        return item

    @staticmethod
    def create_pcs():
        """
            Создать единицу "штука"
        Returns:
            Модель штуки
        """
        return measurement_unit_model("штук")

    @staticmethod
    def create_ml():
        """
            Создать единицу "миллилитр"
        Returns:
            Модель миллилитра
        """
        return measurement_unit_model("мл")

    @staticmethod
    def create_l():
        """
            Создать единицу "литр"
        Returns:
            Модель литра
        """
        return measurement_unit_model("литр", 1000, measurement_unit_model.create_ml())
