from src.models.recipe_model import recipe_model
from src.models.measurement_unit_model import measurement_unit_model
from src.models.nomenclature_group_model import nomenclature_group_model
from src.models.nomenclature_model import nomenclature_model
from src.storage.storage_service import storage_service
from src.logic.start_factory import start_factory
from src.settings.settings_manager import settings_manager
from src.storage.storage import storage
from datetime import datetime
import unittest


class test_models(unittest.TestCase):
    def test_period_turns(self):
        options = settings_manager()
        start = start_factory(options.settings, storage())
        start.create()

        start_date = datetime.strptime("2023-01-01", "%Y-%m-%d")
        stop_date = datetime.strptime("2024-10-01", "%Y-%m-%d")

        service = storage_service(start.storage.data[storage.process_turn_key()])
        result = service.create_turns_dt(storage(), start_date, stop_date)

        assert result is not None

    def test_nomen_turns(self):
        options = settings_manager()
        start = start_factory(options.settings, storage())
        start.create()

        nomen = nomenclature_model(
            "sugar",
            "Сахар",
            measurement_unit_model.create_g(),
            nomenclature_group_model.create_group(),
        )

        service = storage_service(start.storage.data[storage.transaction_key()])
        result = service.create_turns_nomen(storage(), nomen)

        assert result is not None

    def test_recipe_turns(self):
        options = settings_manager()
        start = start_factory(options.settings, storage())
        start.create()

        recipe = recipe_model("Test")

        service = storage_service(start.storage.data[storage.transaction_key()])
        result = service.create_turns_recipe(storage(), recipe)

        assert result is not None
